/*
 * SPDX-License-Identifier: ODbL-1.0
 * SPDX-FileCopyrightText: OpenStreetMap contributors
 *
 * Autogenerated using QGIS - do not edit!
 */

#include "isocodes_p.h"
#include "mapentry_p.h"
#include "timezone_names_p.h"

static constexpr const MapEntry<uint16_t> timezone_country_map[] = {
    {Tz::Africa_Abidjan, IsoCodes::alpha2CodeToKey("CI")},
    {Tz::Africa_Accra, IsoCodes::alpha2CodeToKey("GH")},
    {Tz::Africa_Addis_Ababa, IsoCodes::alpha2CodeToKey("ET")},
    {Tz::Africa_Algiers, IsoCodes::alpha2CodeToKey("DZ")},
    {Tz::Africa_Asmara, IsoCodes::alpha2CodeToKey("ER")},
    {Tz::Africa_Bamako, IsoCodes::alpha2CodeToKey("ML")},
    {Tz::Africa_Bangui, IsoCodes::alpha2CodeToKey("CF")},
    {Tz::Africa_Banjul, IsoCodes::alpha2CodeToKey("GM")},
    {Tz::Africa_Bissau, IsoCodes::alpha2CodeToKey("GW")},
    {Tz::Africa_Blantyre, IsoCodes::alpha2CodeToKey("MW")},
    {Tz::Africa_Brazzaville, IsoCodes::alpha2CodeToKey("CG")},
    {Tz::Africa_Bujumbura, IsoCodes::alpha2CodeToKey("BI")},
    {Tz::Africa_Cairo, IsoCodes::alpha2CodeToKey("EG")},
    {Tz::Africa_Casablanca, IsoCodes::alpha2CodeToKey("MA")},
    {Tz::Africa_Ceuta, IsoCodes::alpha2CodeToKey("ES")},
    {Tz::Africa_Conakry, IsoCodes::alpha2CodeToKey("GN")},
    {Tz::Africa_Dakar, IsoCodes::alpha2CodeToKey("SN")},
    {Tz::Africa_Dar_es_Salaam, IsoCodes::alpha2CodeToKey("TZ")},
    {Tz::Africa_Djibouti, IsoCodes::alpha2CodeToKey("DJ")},
    {Tz::Africa_Douala, IsoCodes::alpha2CodeToKey("CM")},
    // Tz::Africa_El_Aaiun
    {Tz::Africa_Freetown, IsoCodes::alpha2CodeToKey("SL")},
    {Tz::Africa_Gaborone, IsoCodes::alpha2CodeToKey("BW")},
    {Tz::Africa_Harare, IsoCodes::alpha2CodeToKey("ZW")},
    {Tz::Africa_Johannesburg, IsoCodes::alpha2CodeToKey("ZA")},
    // Tz::Africa_Juba
    {Tz::Africa_Kampala, IsoCodes::alpha2CodeToKey("UG")},
    // Tz::Africa_Khartoum
    {Tz::Africa_Kigali, IsoCodes::alpha2CodeToKey("RW")},
    {Tz::Africa_Kinshasa, IsoCodes::alpha2CodeToKey("CD")},
    {Tz::Africa_Lagos, IsoCodes::alpha2CodeToKey("NG")},
    {Tz::Africa_Libreville, IsoCodes::alpha2CodeToKey("GA")},
    {Tz::Africa_Lome, IsoCodes::alpha2CodeToKey("TG")},
    {Tz::Africa_Luanda, IsoCodes::alpha2CodeToKey("AO")},
    {Tz::Africa_Lubumbashi, IsoCodes::alpha2CodeToKey("CD")},
    {Tz::Africa_Lusaka, IsoCodes::alpha2CodeToKey("ZM")},
    {Tz::Africa_Malabo, IsoCodes::alpha2CodeToKey("GQ")},
    {Tz::Africa_Maputo, IsoCodes::alpha2CodeToKey("MZ")},
    {Tz::Africa_Maseru, IsoCodes::alpha2CodeToKey("LS")},
    {Tz::Africa_Mbabane, IsoCodes::alpha2CodeToKey("SZ")},
    {Tz::Africa_Mogadishu, IsoCodes::alpha2CodeToKey("SO")},
    {Tz::Africa_Monrovia, IsoCodes::alpha2CodeToKey("LR")},
    {Tz::Africa_Nairobi, IsoCodes::alpha2CodeToKey("KE")},
    {Tz::Africa_Ndjamena, IsoCodes::alpha2CodeToKey("TD")},
    {Tz::Africa_Niamey, IsoCodes::alpha2CodeToKey("NE")},
    {Tz::Africa_Nouakchott, IsoCodes::alpha2CodeToKey("MR")},
    {Tz::Africa_Ouagadougou, IsoCodes::alpha2CodeToKey("BF")},
    {Tz::Africa_Porto_Novo, IsoCodes::alpha2CodeToKey("BJ")},
    {Tz::Africa_Sao_Tome, IsoCodes::alpha2CodeToKey("ST")},
    {Tz::Africa_Tripoli, IsoCodes::alpha2CodeToKey("LY")},
    {Tz::Africa_Tunis, IsoCodes::alpha2CodeToKey("TN")},
    {Tz::Africa_Windhoek, IsoCodes::alpha2CodeToKey("NA")},
    {Tz::America_Adak, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Anchorage, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Anguilla, IsoCodes::alpha2CodeToKey("AI")},
    {Tz::America_Antigua, IsoCodes::alpha2CodeToKey("AG")},
    {Tz::America_Araguaina, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Argentina_Buenos_Aires, IsoCodes::alpha2CodeToKey("AR")},
    {Tz::America_Argentina_Catamarca, IsoCodes::alpha2CodeToKey("AR")},
    {Tz::America_Argentina_Cordoba, IsoCodes::alpha2CodeToKey("AR")},
    {Tz::America_Argentina_Jujuy, IsoCodes::alpha2CodeToKey("AR")},
    {Tz::America_Argentina_La_Rioja, IsoCodes::alpha2CodeToKey("AR")},
    {Tz::America_Argentina_Mendoza, IsoCodes::alpha2CodeToKey("AR")},
    {Tz::America_Argentina_Rio_Gallegos, IsoCodes::alpha2CodeToKey("AR")},
    {Tz::America_Argentina_Salta, IsoCodes::alpha2CodeToKey("AR")},
    {Tz::America_Argentina_San_Juan, IsoCodes::alpha2CodeToKey("AR")},
    {Tz::America_Argentina_San_Luis, IsoCodes::alpha2CodeToKey("AR")},
    {Tz::America_Argentina_Tucuman, IsoCodes::alpha2CodeToKey("AR")},
    {Tz::America_Argentina_Ushuaia, IsoCodes::alpha2CodeToKey("AR")},
    {Tz::America_Aruba, IsoCodes::alpha2CodeToKey("AW")},
    {Tz::America_Asuncion, IsoCodes::alpha2CodeToKey("PY")},
    {Tz::America_Atikokan, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Bahia, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Bahia_Banderas, IsoCodes::alpha2CodeToKey("MX")},
    {Tz::America_Barbados, IsoCodes::alpha2CodeToKey("BB")},
    {Tz::America_Belem, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Belize, IsoCodes::alpha2CodeToKey("BZ")},
    {Tz::America_Blanc_Sablon, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Boa_Vista, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Bogota, IsoCodes::alpha2CodeToKey("CO")},
    {Tz::America_Boise, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Cambridge_Bay, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Campo_Grande, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Cancun, IsoCodes::alpha2CodeToKey("MX")},
    {Tz::America_Caracas, IsoCodes::alpha2CodeToKey("VE")},
    {Tz::America_Cayenne, IsoCodes::alpha2CodeToKey("GF")},
    {Tz::America_Cayman, IsoCodes::alpha2CodeToKey("KY")},
    {Tz::America_Chicago, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Chihuahua, IsoCodes::alpha2CodeToKey("MX")},
    {Tz::America_Costa_Rica, IsoCodes::alpha2CodeToKey("CR")},
    {Tz::America_Creston, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Cuiaba, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Curacao, IsoCodes::alpha2CodeToKey("CW")},
    {Tz::America_Danmarkshavn, IsoCodes::alpha2CodeToKey("GL")},
    {Tz::America_Dawson, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Dawson_Creek, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Denver, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Detroit, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Dominica, IsoCodes::alpha2CodeToKey("DM")},
    {Tz::America_Edmonton, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Eirunepe, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_El_Salvador, IsoCodes::alpha2CodeToKey("SV")},
    {Tz::America_Fort_Nelson, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Fortaleza, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Glace_Bay, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Goose_Bay, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Grand_Turk, IsoCodes::alpha2CodeToKey("TC")},
    {Tz::America_Grenada, IsoCodes::alpha2CodeToKey("GD")},
    {Tz::America_Guadeloupe, IsoCodes::alpha2CodeToKey("GP")},
    {Tz::America_Guatemala, IsoCodes::alpha2CodeToKey("GT")},
    {Tz::America_Guayaquil, IsoCodes::alpha2CodeToKey("EC")},
    {Tz::America_Guyana, IsoCodes::alpha2CodeToKey("GY")},
    {Tz::America_Halifax, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Havana, IsoCodes::alpha2CodeToKey("CU")},
    {Tz::America_Hermosillo, IsoCodes::alpha2CodeToKey("MX")},
    {Tz::America_Indiana_Indianapolis, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Indiana_Knox, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Indiana_Marengo, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Indiana_Petersburg, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Indiana_Tell_City, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Indiana_Vevay, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Indiana_Vincennes, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Indiana_Winamac, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Inuvik, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Iqaluit, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Jamaica, IsoCodes::alpha2CodeToKey("JM")},
    {Tz::America_Juneau, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Kentucky_Louisville, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Kentucky_Monticello, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Kralendijk, IsoCodes::alpha2CodeToKey("BQ")},
    {Tz::America_La_Paz, IsoCodes::alpha2CodeToKey("BO")},
    {Tz::America_Lima, IsoCodes::alpha2CodeToKey("PE")},
    {Tz::America_Los_Angeles, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Lower_Princes, IsoCodes::alpha2CodeToKey("SX")},
    {Tz::America_Maceio, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Managua, IsoCodes::alpha2CodeToKey("NI")},
    {Tz::America_Manaus, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Marigot, IsoCodes::alpha2CodeToKey("MF")},
    {Tz::America_Martinique, IsoCodes::alpha2CodeToKey("MQ")},
    {Tz::America_Matamoros, IsoCodes::alpha2CodeToKey("MX")},
    {Tz::America_Mazatlan, IsoCodes::alpha2CodeToKey("MX")},
    {Tz::America_Menominee, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Merida, IsoCodes::alpha2CodeToKey("MX")},
    {Tz::America_Metlakatla, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Mexico_City, IsoCodes::alpha2CodeToKey("MX")},
    {Tz::America_Miquelon, IsoCodes::alpha2CodeToKey("PM")},
    {Tz::America_Moncton, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Monterrey, IsoCodes::alpha2CodeToKey("MX")},
    {Tz::America_Montevideo, IsoCodes::alpha2CodeToKey("UY")},
    {Tz::America_Montserrat, IsoCodes::alpha2CodeToKey("MS")},
    {Tz::America_Nassau, IsoCodes::alpha2CodeToKey("BS")},
    {Tz::America_New_York, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Nipigon, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Nome, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Noronha, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_North_Dakota_Beulah, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_North_Dakota_Center, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_North_Dakota_New_Salem, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Nuuk, IsoCodes::alpha2CodeToKey("GL")},
    {Tz::America_Ojinaga, IsoCodes::alpha2CodeToKey("MX")},
    {Tz::America_Panama, IsoCodes::alpha2CodeToKey("PA")},
    {Tz::America_Pangnirtung, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Paramaribo, IsoCodes::alpha2CodeToKey("SR")},
    {Tz::America_Phoenix, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Port_au_Prince, IsoCodes::alpha2CodeToKey("HT")},
    {Tz::America_Port_of_Spain, IsoCodes::alpha2CodeToKey("TT")},
    {Tz::America_Porto_Velho, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Puerto_Rico, IsoCodes::alpha2CodeToKey("PR")},
    {Tz::America_Punta_Arenas, IsoCodes::alpha2CodeToKey("CL")},
    {Tz::America_Rainy_River, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Rankin_Inlet, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Recife, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Regina, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Resolute, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Rio_Branco, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Santarem, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Santiago, IsoCodes::alpha2CodeToKey("CL")},
    {Tz::America_Santo_Domingo, IsoCodes::alpha2CodeToKey("DO")},
    {Tz::America_Sao_Paulo, IsoCodes::alpha2CodeToKey("BR")},
    {Tz::America_Scoresbysund, IsoCodes::alpha2CodeToKey("GL")},
    {Tz::America_Sitka, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_St_Barthelemy, IsoCodes::alpha2CodeToKey("BL")},
    {Tz::America_St_Johns, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_St_Kitts, IsoCodes::alpha2CodeToKey("KN")},
    {Tz::America_St_Lucia, IsoCodes::alpha2CodeToKey("LC")},
    {Tz::America_St_Thomas, IsoCodes::alpha2CodeToKey("VI")},
    {Tz::America_St_Vincent, IsoCodes::alpha2CodeToKey("VC")},
    {Tz::America_Swift_Current, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Tegucigalpa, IsoCodes::alpha2CodeToKey("HN")},
    {Tz::America_Thule, IsoCodes::alpha2CodeToKey("GL")},
    {Tz::America_Thunder_Bay, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Tijuana, IsoCodes::alpha2CodeToKey("MX")},
    {Tz::America_Toronto, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Tortola, IsoCodes::alpha2CodeToKey("VG")},
    {Tz::America_Vancouver, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Whitehorse, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Winnipeg, IsoCodes::alpha2CodeToKey("CA")},
    {Tz::America_Yakutat, IsoCodes::alpha2CodeToKey("US")},
    {Tz::America_Yellowknife, IsoCodes::alpha2CodeToKey("CA")},
    // Tz::Antarctica_Casey
    // Tz::Antarctica_Davis
    // Tz::Antarctica_DumontDUrville
    {Tz::Antarctica_Macquarie, IsoCodes::alpha2CodeToKey("AU")},
    // Tz::Antarctica_Mawson
    // Tz::Antarctica_McMurdo
    // Tz::Antarctica_Palmer
    // Tz::Antarctica_Rothera
    // Tz::Antarctica_Syowa
    // Tz::Antarctica_Troll
    // Tz::Antarctica_Vostok
    {Tz::Arctic_Longyearbyen, IsoCodes::alpha2CodeToKey("NO")},
    {Tz::Asia_Aden, IsoCodes::alpha2CodeToKey("YE")},
    {Tz::Asia_Almaty, IsoCodes::alpha2CodeToKey("KZ")},
    {Tz::Asia_Amman, IsoCodes::alpha2CodeToKey("JO")},
    {Tz::Asia_Anadyr, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Aqtau, IsoCodes::alpha2CodeToKey("KZ")},
    {Tz::Asia_Aqtobe, IsoCodes::alpha2CodeToKey("KZ")},
    {Tz::Asia_Ashgabat, IsoCodes::alpha2CodeToKey("TM")},
    {Tz::Asia_Atyrau, IsoCodes::alpha2CodeToKey("KZ")},
    {Tz::Asia_Baghdad, IsoCodes::alpha2CodeToKey("IQ")},
    {Tz::Asia_Bahrain, IsoCodes::alpha2CodeToKey("BH")},
    {Tz::Asia_Baku, IsoCodes::alpha2CodeToKey("AZ")},
    // Tz::Asia_Bangkok
    {Tz::Asia_Barnaul, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Beirut, IsoCodes::alpha2CodeToKey("LB")},
    {Tz::Asia_Bishkek, IsoCodes::alpha2CodeToKey("KG")},
    {Tz::Asia_Brunei, IsoCodes::alpha2CodeToKey("BN")},
    {Tz::Asia_Chita, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Choibalsan, IsoCodes::alpha2CodeToKey("MN")},
    {Tz::Asia_Colombo, IsoCodes::alpha2CodeToKey("LK")},
    {Tz::Asia_Damascus, IsoCodes::alpha2CodeToKey("SY")},
    {Tz::Asia_Dhaka, IsoCodes::alpha2CodeToKey("BD")},
    // Tz::Asia_Dili
    {Tz::Asia_Dubai, IsoCodes::alpha2CodeToKey("AE")},
    {Tz::Asia_Dushanbe, IsoCodes::alpha2CodeToKey("TJ")},
    {Tz::Asia_Famagusta, IsoCodes::alpha2CodeToKey("CY")},
    {Tz::Asia_Gaza, IsoCodes::alpha2CodeToKey("PS")},
    {Tz::Asia_Hebron, IsoCodes::alpha2CodeToKey("PS")},
    {Tz::Asia_Ho_Chi_Minh, IsoCodes::alpha2CodeToKey("VN")},
    {Tz::Asia_Hong_Kong, IsoCodes::alpha2CodeToKey("HK")},
    {Tz::Asia_Hovd, IsoCodes::alpha2CodeToKey("MN")},
    {Tz::Asia_Irkutsk, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Jakarta, IsoCodes::alpha2CodeToKey("ID")},
    {Tz::Asia_Jayapura, IsoCodes::alpha2CodeToKey("ID")},
    // Tz::Asia_Jerusalem
    {Tz::Asia_Kabul, IsoCodes::alpha2CodeToKey("AF")},
    {Tz::Asia_Kamchatka, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Karachi, IsoCodes::alpha2CodeToKey("PK")},
    {Tz::Asia_Kathmandu, IsoCodes::alpha2CodeToKey("NP")},
    {Tz::Asia_Khandyga, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Kolkata, IsoCodes::alpha2CodeToKey("IN")},
    {Tz::Asia_Krasnoyarsk, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Kuala_Lumpur, IsoCodes::alpha2CodeToKey("MY")},
    {Tz::Asia_Kuching, IsoCodes::alpha2CodeToKey("MY")},
    {Tz::Asia_Kuwait, IsoCodes::alpha2CodeToKey("KW")},
    {Tz::Asia_Macau, IsoCodes::alpha2CodeToKey("MO")},
    {Tz::Asia_Magadan, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Makassar, IsoCodes::alpha2CodeToKey("ID")},
    {Tz::Asia_Manila, IsoCodes::alpha2CodeToKey("PH")},
    {Tz::Asia_Muscat, IsoCodes::alpha2CodeToKey("OM")},
    {Tz::Asia_Nicosia, IsoCodes::alpha2CodeToKey("CY")},
    {Tz::Asia_Novokuznetsk, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Novosibirsk, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Omsk, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Oral, IsoCodes::alpha2CodeToKey("KZ")},
    {Tz::Asia_Phnom_Penh, IsoCodes::alpha2CodeToKey("KH")},
    {Tz::Asia_Pontianak, IsoCodes::alpha2CodeToKey("ID")},
    {Tz::Asia_Pyongyang, IsoCodes::alpha2CodeToKey("KP")},
    {Tz::Asia_Qatar, IsoCodes::alpha2CodeToKey("QA")},
    {Tz::Asia_Qostanay, IsoCodes::alpha2CodeToKey("KZ")},
    {Tz::Asia_Qyzylorda, IsoCodes::alpha2CodeToKey("KZ")},
    {Tz::Asia_Riyadh, IsoCodes::alpha2CodeToKey("SA")},
    {Tz::Asia_Sakhalin, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Samarkand, IsoCodes::alpha2CodeToKey("UZ")},
    {Tz::Asia_Seoul, IsoCodes::alpha2CodeToKey("KR")},
    {Tz::Asia_Shanghai, IsoCodes::alpha2CodeToKey("CN")},
    {Tz::Asia_Singapore, IsoCodes::alpha2CodeToKey("SG")},
    {Tz::Asia_Srednekolymsk, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Taipei, IsoCodes::alpha2CodeToKey("TW")},
    {Tz::Asia_Tashkent, IsoCodes::alpha2CodeToKey("UZ")},
    {Tz::Asia_Tbilisi, IsoCodes::alpha2CodeToKey("GE")},
    {Tz::Asia_Tehran, IsoCodes::alpha2CodeToKey("IR")},
    {Tz::Asia_Thimphu, IsoCodes::alpha2CodeToKey("BT")},
    {Tz::Asia_Tokyo, IsoCodes::alpha2CodeToKey("JP")},
    {Tz::Asia_Tomsk, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Ulaanbaatar, IsoCodes::alpha2CodeToKey("MN")},
    {Tz::Asia_Urumqi, IsoCodes::alpha2CodeToKey("CN")},
    {Tz::Asia_Ust_Nera, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Vientiane, IsoCodes::alpha2CodeToKey("LA")},
    {Tz::Asia_Vladivostok, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Yakutsk, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Yangon, IsoCodes::alpha2CodeToKey("MM")},
    {Tz::Asia_Yekaterinburg, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Asia_Yerevan, IsoCodes::alpha2CodeToKey("AM")},
    {Tz::Atlantic_Azores, IsoCodes::alpha2CodeToKey("PT")},
    {Tz::Atlantic_Bermuda, IsoCodes::alpha2CodeToKey("BM")},
    {Tz::Atlantic_Canary, IsoCodes::alpha2CodeToKey("ES")},
    {Tz::Atlantic_Cape_Verde, IsoCodes::alpha2CodeToKey("CV")},
    {Tz::Atlantic_Faroe, IsoCodes::alpha2CodeToKey("FO")},
    {Tz::Atlantic_Madeira, IsoCodes::alpha2CodeToKey("PT")},
    {Tz::Atlantic_Reykjavik, IsoCodes::alpha2CodeToKey("IS")},
    {Tz::Atlantic_South_Georgia, IsoCodes::alpha2CodeToKey("GS")},
    {Tz::Atlantic_St_Helena, IsoCodes::alpha2CodeToKey("SH")},
    {Tz::Atlantic_Stanley, IsoCodes::alpha2CodeToKey("FK")},
    {Tz::Australia_Adelaide, IsoCodes::alpha2CodeToKey("AU")},
    {Tz::Australia_Brisbane, IsoCodes::alpha2CodeToKey("AU")},
    {Tz::Australia_Broken_Hill, IsoCodes::alpha2CodeToKey("AU")},
    {Tz::Australia_Currie, IsoCodes::alpha2CodeToKey("AU")},
    {Tz::Australia_Darwin, IsoCodes::alpha2CodeToKey("AU")},
    {Tz::Australia_Eucla, IsoCodes::alpha2CodeToKey("AU")},
    {Tz::Australia_Hobart, IsoCodes::alpha2CodeToKey("AU")},
    {Tz::Australia_Lindeman, IsoCodes::alpha2CodeToKey("AU")},
    {Tz::Australia_Lord_Howe, IsoCodes::alpha2CodeToKey("AU")},
    {Tz::Australia_Melbourne, IsoCodes::alpha2CodeToKey("AU")},
    {Tz::Australia_Perth, IsoCodes::alpha2CodeToKey("AU")},
    {Tz::Australia_Sydney, IsoCodes::alpha2CodeToKey("AU")},
    // Tz::Etc_UTC
    {Tz::Europe_Amsterdam, IsoCodes::alpha2CodeToKey("NL")},
    {Tz::Europe_Andorra, IsoCodes::alpha2CodeToKey("AD")},
    {Tz::Europe_Astrakhan, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Europe_Athens, IsoCodes::alpha2CodeToKey("GR")},
    // Tz::Europe_Belgrade
    {Tz::Europe_Berlin, IsoCodes::alpha2CodeToKey("DE")},
    {Tz::Europe_Bratislava, IsoCodes::alpha2CodeToKey("SK")},
    {Tz::Europe_Brussels, IsoCodes::alpha2CodeToKey("BE")},
    {Tz::Europe_Bucharest, IsoCodes::alpha2CodeToKey("RO")},
    {Tz::Europe_Budapest, IsoCodes::alpha2CodeToKey("HU")},
    {Tz::Europe_Busingen, IsoCodes::alpha2CodeToKey("DE")},
    {Tz::Europe_Chisinau, IsoCodes::alpha2CodeToKey("MD")},
    {Tz::Europe_Copenhagen, IsoCodes::alpha2CodeToKey("DK")},
    {Tz::Europe_Dublin, IsoCodes::alpha2CodeToKey("IE")},
    {Tz::Europe_Gibraltar, IsoCodes::alpha2CodeToKey("GI")},
    {Tz::Europe_Guernsey, IsoCodes::alpha2CodeToKey("GG")},
    {Tz::Europe_Helsinki, IsoCodes::alpha2CodeToKey("FI")},
    {Tz::Europe_Isle_of_Man, IsoCodes::alpha2CodeToKey("IM")},
    {Tz::Europe_Istanbul, IsoCodes::alpha2CodeToKey("TR")},
    {Tz::Europe_Jersey, IsoCodes::alpha2CodeToKey("JE")},
    {Tz::Europe_Kaliningrad, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Europe_Kiev, IsoCodes::alpha2CodeToKey("UA")},
    {Tz::Europe_Kirov, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Europe_Lisbon, IsoCodes::alpha2CodeToKey("PT")},
    {Tz::Europe_Ljubljana, IsoCodes::alpha2CodeToKey("SI")},
    {Tz::Europe_London, IsoCodes::alpha2CodeToKey("GB")},
    {Tz::Europe_Luxembourg, IsoCodes::alpha2CodeToKey("LU")},
    {Tz::Europe_Madrid, IsoCodes::alpha2CodeToKey("ES")},
    {Tz::Europe_Malta, IsoCodes::alpha2CodeToKey("MT")},
    {Tz::Europe_Mariehamn, IsoCodes::alpha2CodeToKey("FI")},
    {Tz::Europe_Minsk, IsoCodes::alpha2CodeToKey("BY")},
    {Tz::Europe_Monaco, IsoCodes::alpha2CodeToKey("MC")},
    // Tz::Europe_Moscow
    {Tz::Europe_Oslo, IsoCodes::alpha2CodeToKey("NO")},
    {Tz::Europe_Paris, IsoCodes::alpha2CodeToKey("FR")},
    {Tz::Europe_Podgorica, IsoCodes::alpha2CodeToKey("ME")},
    {Tz::Europe_Prague, IsoCodes::alpha2CodeToKey("CZ")},
    {Tz::Europe_Riga, IsoCodes::alpha2CodeToKey("LV")},
    {Tz::Europe_Rome, IsoCodes::alpha2CodeToKey("IT")},
    {Tz::Europe_Samara, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Europe_San_Marino, IsoCodes::alpha2CodeToKey("SM")},
    {Tz::Europe_Sarajevo, IsoCodes::alpha2CodeToKey("BA")},
    {Tz::Europe_Saratov, IsoCodes::alpha2CodeToKey("RU")},
    // Tz::Europe_Simferopol
    {Tz::Europe_Skopje, IsoCodes::alpha2CodeToKey("MK")},
    {Tz::Europe_Sofia, IsoCodes::alpha2CodeToKey("BG")},
    {Tz::Europe_Stockholm, IsoCodes::alpha2CodeToKey("SE")},
    {Tz::Europe_Tallinn, IsoCodes::alpha2CodeToKey("EE")},
    {Tz::Europe_Tirane, IsoCodes::alpha2CodeToKey("AL")},
    {Tz::Europe_Ulyanovsk, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Europe_Uzhgorod, IsoCodes::alpha2CodeToKey("UA")},
    {Tz::Europe_Vaduz, IsoCodes::alpha2CodeToKey("LI")},
    {Tz::Europe_Vatican, IsoCodes::alpha2CodeToKey("VA")},
    {Tz::Europe_Vienna, IsoCodes::alpha2CodeToKey("AT")},
    {Tz::Europe_Vilnius, IsoCodes::alpha2CodeToKey("LT")},
    {Tz::Europe_Volgograd, IsoCodes::alpha2CodeToKey("RU")},
    {Tz::Europe_Warsaw, IsoCodes::alpha2CodeToKey("PL")},
    {Tz::Europe_Zagreb, IsoCodes::alpha2CodeToKey("HR")},
    {Tz::Europe_Zaporozhye, IsoCodes::alpha2CodeToKey("UA")},
    {Tz::Europe_Zurich, IsoCodes::alpha2CodeToKey("CH")},
    {Tz::Indian_Antananarivo, IsoCodes::alpha2CodeToKey("MG")},
    {Tz::Indian_Chagos, IsoCodes::alpha2CodeToKey("IO")},
    {Tz::Indian_Christmas, IsoCodes::alpha2CodeToKey("CX")},
    {Tz::Indian_Cocos, IsoCodes::alpha2CodeToKey("CC")},
    {Tz::Indian_Comoro, IsoCodes::alpha2CodeToKey("KM")},
    {Tz::Indian_Kerguelen, IsoCodes::alpha2CodeToKey("TF")},
    {Tz::Indian_Mahe, IsoCodes::alpha2CodeToKey("SC")},
    {Tz::Indian_Maldives, IsoCodes::alpha2CodeToKey("MV")},
    {Tz::Indian_Mauritius, IsoCodes::alpha2CodeToKey("MU")},
    {Tz::Indian_Mayotte, IsoCodes::alpha2CodeToKey("YT")},
    {Tz::Indian_Reunion, IsoCodes::alpha2CodeToKey("RE")},
    {Tz::Pacific_Apia, IsoCodes::alpha2CodeToKey("WS")},
    {Tz::Pacific_Auckland, IsoCodes::alpha2CodeToKey("NZ")},
    {Tz::Pacific_Bougainville, IsoCodes::alpha2CodeToKey("PG")},
    {Tz::Pacific_Chatham, IsoCodes::alpha2CodeToKey("NZ")},
    {Tz::Pacific_Chuuk, IsoCodes::alpha2CodeToKey("FM")},
    {Tz::Pacific_Easter, IsoCodes::alpha2CodeToKey("CL")},
    {Tz::Pacific_Efate, IsoCodes::alpha2CodeToKey("VU")},
    {Tz::Pacific_Enderbury, IsoCodes::alpha2CodeToKey("KI")},
    {Tz::Pacific_Fakaofo, IsoCodes::alpha2CodeToKey("TK")},
    {Tz::Pacific_Fiji, IsoCodes::alpha2CodeToKey("FJ")},
    {Tz::Pacific_Funafuti, IsoCodes::alpha2CodeToKey("TV")},
    {Tz::Pacific_Galapagos, IsoCodes::alpha2CodeToKey("EC")},
    {Tz::Pacific_Gambier, IsoCodes::alpha2CodeToKey("PF")},
    {Tz::Pacific_Guadalcanal, IsoCodes::alpha2CodeToKey("SB")},
    {Tz::Pacific_Guam, IsoCodes::alpha2CodeToKey("GU")},
    {Tz::Pacific_Honolulu, IsoCodes::alpha2CodeToKey("US")},
    {Tz::Pacific_Kiritimati, IsoCodes::alpha2CodeToKey("KI")},
    {Tz::Pacific_Kosrae, IsoCodes::alpha2CodeToKey("FM")},
    {Tz::Pacific_Kwajalein, IsoCodes::alpha2CodeToKey("MH")},
    {Tz::Pacific_Majuro, IsoCodes::alpha2CodeToKey("MH")},
    {Tz::Pacific_Marquesas, IsoCodes::alpha2CodeToKey("PF")},
    {Tz::Pacific_Midway, IsoCodes::alpha2CodeToKey("US")},
    {Tz::Pacific_Nauru, IsoCodes::alpha2CodeToKey("NR")},
    {Tz::Pacific_Niue, IsoCodes::alpha2CodeToKey("NU")},
    {Tz::Pacific_Norfolk, IsoCodes::alpha2CodeToKey("NF")},
    {Tz::Pacific_Noumea, IsoCodes::alpha2CodeToKey("NC")},
    {Tz::Pacific_Pago_Pago, IsoCodes::alpha2CodeToKey("AS")},
    {Tz::Pacific_Palau, IsoCodes::alpha2CodeToKey("PW")},
    {Tz::Pacific_Pitcairn, IsoCodes::alpha2CodeToKey("PN")},
    {Tz::Pacific_Pohnpei, IsoCodes::alpha2CodeToKey("FM")},
    {Tz::Pacific_Port_Moresby, IsoCodes::alpha2CodeToKey("PG")},
    {Tz::Pacific_Rarotonga, IsoCodes::alpha2CodeToKey("CK")},
    {Tz::Pacific_Saipan, IsoCodes::alpha2CodeToKey("MP")},
    {Tz::Pacific_Tahiti, IsoCodes::alpha2CodeToKey("PF")},
    {Tz::Pacific_Tarawa, IsoCodes::alpha2CodeToKey("KI")},
    {Tz::Pacific_Tongatapu, IsoCodes::alpha2CodeToKey("TO")},
    {Tz::Pacific_Wake, IsoCodes::alpha2CodeToKey("US")},
    {Tz::Pacific_Wallis, IsoCodes::alpha2CodeToKey("WF")},
};
